# flake8: noqa
import asyncio
import os

from .application import Application
from .response import JSONResponse, Response, HTMLResponse, JavaScriptResponse
from .view import View
from .resource import Resource


@asyncio.coroutine
def init(loop, cls, routes, *args, **kwargs): # pragma: no cover

    app = cls(*args, loop=loop, routes=routes, **kwargs)

    HOST = os.getenv('HOST', '0.0.0.0')
    PORT = os.getenv('PORT', 8000)

    srv = yield from loop.create_server(app.make_handler(), HOST, PORT)

    return srv


def run(cls, routes, *args, **kwargs): # pragma: no cover
    """
    Run a web application.

    :param cls: Application class
    :param routes: list of routes
    :param args: additional arguments
    :param kwargs: additional keyword arguments
    :return: None
    """
    loop = asyncio.get_event_loop()
    loop.run_until_complete(init(loop, cls, routes, *args, **kwargs))

    try:
        loop.run_forever()
    except KeyboardInterrupt as e:
        loop.close()

    print('Process finished.')



from aiohttp import request
from chilero import web
from chilero.web.test import WebTestCase, asynctest


class HelloView(web.View):

    def get(self):
        return web.Response('Hello world!')

class MultiView(web.View):

    def get(self, type):

        types = dict(
            plain=lambda: web.Response('Hello world!'),
            html=lambda: web.HTMLResponse('<h1>Hello world!</h1>'),
            json=lambda: web.JSONResponse(dict(hello='world!')),
            javascript=lambda: web.JavaScriptResponse('// hello world!'),
        )

        return types.get(type, 'plain')()


class TestWeb(WebTestCase):
    routes = [
        ['/hello/', HelloView],
        ['/{type}', MultiView]
    ]

    @asynctest
    def test_hello_view(self):

        resp = yield from request(
            'GET',self.full_url('/hello'),loop=self.loop
        )
        print(dir(resp))
        self.assertEqual(resp.status, 200)


    @asynctest
    def test_multi_view(self):

        types = dict(
            plain=[None, 'Hello world!'],
            html=['text/html', '<h1>Hello world!</h1>'],
            javascript=['application/javascript', '// hello world!'],
            json=['text/json', '{"hello": "world!"}']
        )

        for t in types.keys():
            resp = yield from request(
                'GET',
                self.full_url(self.app.reverse('multiview', type=t)),
                loop=self.loop
            )

            self.assertEqual(resp.status, 200)

            self.assertEqual(
                types[t][0], resp.headers.get('CONTENT-TYPE'))

            text = yield from resp.text()

            # self.assertEqual(types[t][1], text)



import json

from aiohttp import web


class Response(web.Response):
    def __init__(self, body=None, **kwargs):
        super(Response, self).__init__(
            body=body.encode('utf-8') if body is not None else body, **kwargs
        )


class JavaScriptResponse(Response):

    def __init__(self, data, **kwargs):
        super(JavaScriptResponse, self).__init__(
            body=json.dumps(data), content_type='application/javascript',
            **kwargs
        )


class JSONResponse(Response):

    def __init__(self, data, **kwargs):
        super(JSONResponse, self).__init__(
            body=json.dumps(data, indent=4), content_type='text/json', **kwargs
        )


class HTMLResponse(Response):

    def __init__(self, body, **kwargs):
        super(HTMLResponse, self).__init__(
            body=body, content_type='text/html', **kwargs
        )



import asyncio
import os

from aiohttp import hdrs, web
from aiohttp.web_urldispatcher import UrlDispatcher
from chilero.web.resource import Resource


class Application(web.Application):

    def __init__(self, routes=None, **kwargs):
        super(Application, self).__init__(**kwargs)

        for route in routes or []:
            self.register_routes(route)

    def dispatcher(self, cls, method):
        @asyncio.coroutine
        def f(request, *args, **kwargs):
            vkwargs = dict()
            for k in request.match_info.keys():
                vkwargs[k] = request.match_info.get(k)
            return getattr(
                cls(request, self, *args, **kwargs), method
            )(**vkwargs)

        return f

    def register_routes(self, route):
        pattern = route[0]
        view = route[1]

        # remove trailing slash
        if pattern.endswith('/') and len(pattern) > 1:
            pattern = pattern[:-1]

        if issubclass(view, Resource):
            # Add resource actions as urls

            url_name = route[2] \
                if len(route) == 3 \
                else view.resource_name \
                if hasattr(view, 'resource_name') \
                else view.__name__.lower()

            object_pattern = r'%s' % os.path.join(pattern, view.id_pattern)

            # Nested resources
            for nkey, nview in (
                    view.nested_collection_resources or {}
            ).items():
                self.register_routes(
                    [
                        os.path.join(
                            pattern if pattern != '/' else '',
                            nkey
                        ),
                        nview
                    ]
                )

            for nkey, nview in (view.nested_entity_resources or {}).items():
                self.register_routes(
                    [
                        os.path.join(
                            object_pattern.replace(
                                '{id}', '{%s_id}' % url_name
                            ),
                            nkey
                        ),
                        nview
                    ]
                )

            patterns = {
                # Collection's actions to HTTP methods mapping
                pattern: dict(
                    index=[hdrs.METH_GET],
                    new=[hdrs.METH_POST, hdrs.METH_PUT],
                ),
                # Element's actions to HTTP methods mapping
                object_pattern: dict(
                    show=[hdrs.METH_GET],
                    update=[hdrs.METH_PUT, hdrs.METH_PATCH],
                    destroy=[hdrs.METH_DELETE]
                )

            }

            for pt, actions in patterns.items():
                for action, methods in actions.items():
                    if callable(getattr(view, action, None)):
                        for method in methods:
                            name = '{}_{}'.format(
                                url_name, 'index' if pt == pattern else 'item'
                            )

                            name = None if name in self.router else name

                            self.router.add_route(
                                method, pt, self.dispatcher(view, action),
                                name=name
                            )

        else:
            # Its a basic view
            url_name = route[2] \
                if len(route) == 3 \
                else view.__name__.lower()

            # HTTP methods as lowercase view methods
            for method in UrlDispatcher.METHODS:

                if callable(getattr(view, method.lower(), None)):
                    # Do not bind the same method twice

                    name = url_name

                    name = None if name in self.router else url_name

                    self.router.add_route(
                        method,
                        pattern,
                        self.dispatcher(view, method.lower()),
                        name=name
                    )

    def reverse(self, name, query=None, **kwargs):
        assert name in self.router, "Url '{}' doesn't exists!".format(name)
        if kwargs:
            return self.router[name].url(parts=kwargs, query=query)
        else:
            return self.router[name].url(query=query)



class View(object):
    def __init__(self, request, app, *args, **kwargs):
        self.request = request
        self.app = app
        self.args = args
        self.kwargs = kwargs

    def get_full_url(self, path):
        return '{}://{}{}'.format(
            self.request.scheme, self.request.host, path
        )



import aiohttp
from aiohttp.web_exceptions import HTTPNotFound
import json
from chilero import web
from chilero.web.test import WebTestCase, asynctest

fruits = dict(
    orange=dict(
        colors=['orange', 'yellow', 'green']
    ),
    strawberry=dict(
        colors=['red', 'pink']
    ),
)

class GlobalStatsResource(web.Resource):
    def index(self):
        return self.response(
            dict(
                count=fruits.__len__()
            )
        )

class SingleStatsResource(web.Resource):
    def index(self, fruit_id):
        return self.response(
            dict(
                count=fruits[fruit_id]['colors'].__len__()
            )
        )

class FruitResource(web.Resource):
    resource_name = 'fruit'

    nested_collection_resources = dict(
        stats=GlobalStatsResource
    )
    nested_entity_resources = dict(
        stats=SingleStatsResource
    )

    def index(self):
        response = dict(
            fruits=fruits
        )
        return self.response(response)

    def show(self, id):
        if id not in fruits:
            raise HTTPNotFound()

        return self.response(fruits[id])

    def new(self):
        data = yield from self.request.json()
        fruits[data['name']] = dict(colors=data['colors'])

        return web.JSONResponse(fruits[data['name']])

    def update(self, id):
        data = yield from self.request.json()
        fruits[id] = dict(colors=data['colors'])

        return web.JSONResponse(fruits[id])

    def destroy(self, id):
        fruits.pop(id)
        return web.JSONResponse(None)


class TestResource(WebTestCase):
    routes = [
        ['/fruit', FruitResource]
    ]

    @asynctest
    def test_index(self):
        resp = yield from aiohttp.get(
            self.full_url(self.app.reverse('fruit_index'))
        )

        self.assertEqual(resp.status, 200)
        jr = yield from resp.json()
        self.assertEqual(jr['fruits'], fruits)
        self.assertTrue(jr['self'].endswith('/fruit'))
        resp.close()


    @asynctest
    def test_show(self):
        resp = yield from aiohttp.get(
            self.full_url(self.app.reverse('fruit_item', id='orange'))
        )

        self.assertEqual(resp.status, 200)
        jr = yield from resp.json()

        self.assertEqual(jr['body'], dict(colors=['orange', 'yellow', 'green']))
        resp.close()

        self.assertTrue(jr['self'].endswith('/fruit/orange'))

        resp2 = yield from aiohttp.get(
            self.full_url(self.app.reverse('fruit_item', id='mango'))
        )

        self.assertEqual(resp2.status, 404)
        resp2.close()

    @asynctest
    def test_new(self):

        resp = yield from aiohttp.post(
            self.full_url(self.app.reverse('fruit_index')),
            data=json.dumps(dict(name='apple', colors=['red', 'green'])),
        )

        self.assertEqual(resp.status, 200)

        resp.close()
        resp = yield from aiohttp.get(
            self.full_url(self.app.reverse('fruit_item', id='apple'))
        )

        self.assertEqual(resp.status, 200)
        jr = yield from resp.json()
        self.assertEqual(jr['body'], dict(colors=['red', 'green']))
        resp.close()

    @asynctest
    def test_update(self):

        resp = yield from aiohttp.post(
            self.full_url(self.app.reverse('fruit_index')),
            data=json.dumps(dict(name='pear', colors=['green'])),
        )

        self.assertEqual(resp.status, 200)
        resp.close()

        resp = yield from aiohttp.get(
            self.full_url(self.app.reverse('fruit_item', id='pear'))
        )

        self.assertEqual(resp.status, 200)
        jr = yield from resp.json()
        self.assertEqual(jr['body'], dict(colors=['green']))
        resp.close()

        resp = yield from aiohttp.put(
            self.full_url(self.app.reverse('fruit_item', id='pear')),
            data=json.dumps(dict(colors=['green', 'yellow'])),
        )

        self.assertEqual(resp.status, 200)
        resp.close()

        resp = yield from aiohttp.get(
            self.full_url(self.app.reverse('fruit_item', id='pear'))
        )

        self.assertEqual(resp.status, 200)
        jr = yield from resp.json()
        self.assertEqual(jr['body'], dict(colors=['green', 'yellow']))
        resp.close()

    @asynctest
    def test_delete(self):

        resp = yield from aiohttp.post(
            self.full_url(self.app.reverse('fruit_index')),
            data=json.dumps(dict(name='grape', colors=['purple'])),
        )

        self.assertEqual(resp.status, 200)
        resp.close()

        resp = yield from aiohttp.get(
            self.full_url(self.app.reverse('fruit_item', id='grape'))
        )

        self.assertEqual(resp.status, 200)
        jr = yield from resp.json()

        self.assertEqual(jr['body'], dict(colors=['purple']))
        resp.close()

        resp = yield from aiohttp.delete(
            self.full_url(self.app.reverse('fruit_item', id='grape'))
        )

        self.assertEqual(resp.status, 200)
        resp.close()

        resp = yield from aiohttp.get(
            self.full_url(self.app.reverse('fruit_item', id='grape'))
        )

        self.assertEqual(resp.status, 404)
        resp.close()






import asyncio
import socket
import unittest
from functools import wraps

from aiohttp import log
from chilero import web


def asynctest(f):

    @wraps(f)
    def wrapper(*args, **kwargs):
        @asyncio.coroutine
        def go():
            yield from f(*args, **kwargs)

        loop = asyncio.get_event_loop()
        loop.run_until_complete(go())
    return wrapper


class WebTestCase(unittest.TestCase):
    application = web.Application
    routes = []

    def setUp(self):
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)

        @asyncio.coroutine
        def go():
            self.app = self.application(self.routes, loop=self.loop)
            yield from self.create_server()

        self.loop.run_until_complete(go())

    def tearDown(self):
        self.app.finish()
        self.loop.close()

    @asyncio.coroutine
    def create_server(self):
        if hasattr(self, 'port'):
            return  # pragma: no cover

        app = getattr(self, 'app')
        assert app is not None, 'Application not initialized'

        self.port = self.find_unused_port()

        self.handler = app.make_handler(
            debug=True, keep_alive_on=False,
            access_log=log.access_logger)
        self.server = yield from self.loop.create_server(
            self.handler, '127.0.0.1', self.port)

    def full_url(self, path):
        return 'http://127.0.0.1:{}{}'.format(self.port, path)

    def find_unused_port(self):
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.bind(('127.0.0.1', 0))
        port = s.getsockname()[1]
        s.close()
        return port



from aiohttp import HttpProcessingError


class HttpException(HttpProcessingError):
    pass


class Http400(HttpException):
    code = 400

class Http403(HttpException):
    code = 403

class Http404(HttpException):
    code = 404



from .response import JSONResponse
from .view import View


class Resource(View):
    """
    Base class for HTTP Resources (RESTful?)
    """

    # TODO: Optimize for alphanumeric as base pattern
    id_pattern = '{id}'

    # Allow nested resources
    nested_collection_resources = None
    nested_entity_resources = None

    def response(self, *args, **kwargs):
        if self.is_entity():
            return EntityResponse(self, *args, **kwargs)
        else:
            return CollectionResponse(self, *args, **kwargs)

    def get_resource_name(self):
        return self.resource_name \
            if hasattr(self, 'resource_name') \
            else self.__class__.__name__.lower()

    def get_index_url(self, resource=None, **kwargs):
        return self.get_full_url(
            self.app.reverse(
                '{}_index'.format(resource or self.get_resource_name()),
                **kwargs
            )
        )

    def get_object_url(self, id, resource=None):
        return self.get_full_url(
            self.app.reverse(
                '{}_item'.format(resource or self.get_resource_name()), id=id
            )
        )

    def get_self_url(self):
        resource = self.get_resource_name()
        return self.get_index_url(resource) if self.is_collection() \
            else self.get_object_url(
                self.request.match_info.get('id', None), resource
            )

    def is_entity(self):
        """
        Hacky solution to detect if current instance of the class is an entity
        or collection endpoint.
        """
        return 'id' in self.request.match_info

    def is_collection(self):
        return not self.is_entity()

    def has_nested_resources(self):
        if self.is_collection():
            return True if self.nested_collection_resources else False
        else:
            return True if self.nested_entity_resources else False

    def get_nested_resources(self):
        if self.is_collection():
            return self.nested_collection_resources
        else:
            return self.nested_entity_resources

    def get_nested_urls(self):

        current_args = self.request.match_info.copy()
        if 'id' in current_args:
            id = current_args.pop('id')
            current_args['{}_id'.format(self.get_resource_name())] = id

        return {
            k: self.get_index_url(c.__name__.lower(), **current_args)
            for k, c in self.get_nested_resources().items()
        }


class ResourceResponse(JSONResponse):

    def __init__(self, resource, kind=None, extra_content=None, **kwargs):
        data = dict(
            element='resource:{}'.format(kind),
            self=resource.get_self_url()
        )

        if resource.has_nested_resources():
            data['resources'] = resource.get_nested_urls()

        if extra_content is not None:
            data.update(extra_content)

        super(ResourceResponse, self).__init__(data, **kwargs)


class CollectionResponse(ResourceResponse):

    def __init__(self, resource, extra_content=None, **kwargs):

        super(CollectionResponse, self).__init__(
            resource, 'collection', extra_content=extra_content, **kwargs)


class EntityResponse(ResourceResponse):

    def __init__(self, resource, body=None, extra_content=None, **kwargs):

        if extra_content is None:
            extra_content = {}

        extra_content['body'] = body

        super(EntityResponse, self).__init__(
            resource, 'entity', extra_content=extra_content, **kwargs)



from distutils.core import setup

from setuptools import find_packages

setup(
    name='chilero',
    version='0.2.2',
    namespace_packages=['chilero'],
    packages=find_packages(),
    include_package_data=True,
    url='https://github.com/dmonroy/chilero',
    author='Darwin Monroy',
    author_email='contact@darwinmonroy.com',
    description='A micro framework... A la Tortrix!',
    install_requires=[
        'aiohttp>=0.19',
    ]
)
